
/*
 * Copyright (C) 1998,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
#include <stdio.h>
#include <unistd.h>
#include <limits.h>
#include "cvdev.h"
#include "matrix.h"

main()
{
    int ntrackers, nbuttons;
    int clock_ticks_per_second;
    int sgi_nap_length;
    float hand_position[3],hand_orientation[3];
    float head_position[3],head_orientation[3];
    int have_any_buttons, *buttons;
    int nloops=300;		/* 10 secs worth of events */
    matrix4x4 rotate_hand,rotate_head;
    float valx, valy;

    clock_ticks_per_second = sysconf(_SC_CLK_TCK);
    sgi_nap_length = clock_ticks_per_second / 30; /* want 30fps */
    
    fprintf(stderr," number of clock ticks per second is %d \n",clock_ticks_per_second);
    fprintf(stderr," nap length is %d clock ticks. \n",sgi_nap_length);
    
    if (open_cave_devices(&ntrackers,&nbuttons) < 0)
    {
	fprintf(stderr," problem opening the cave devices. \n");
	exit(1);
    }

    fprintf(stderr, " cave devices online: %d trackers, %d buttons. \n",ntrackers, nbuttons);

    buttons = (int *)malloc(sizeof(int)*nbuttons);

/*    for (;nloops>=0;nloops--) */
    for (;nloops>=0;)
    {
	get_head_data(head_position,head_orientation);
	get_hand_data(hand_position,hand_orientation);
	have_any_buttons = get_buttons(buttons);
	get_valuator(&valx,&valy);

	cv_compute_orientation_matrix(hand_orientation,&rotate_hand);
	cv_compute_orientation_matrix(head_orientation,&rotate_head);
/*
print head position
	fprintf(stderr," %1.3f %1.3f %1.3f  \r",head_position[0], head_position[1], head_position[2]); */

        /* print x and y axes for the head */
/*	fprintf(stderr," %1.3f %1.3f %1.3f   %1.3f %1.3f %1.3f \r",rotate_head.m[0][0], rotate_head.m[1][0], rotate_head.m[2][0],rotate_head.m[0][1], rotate_head.m[1][1], rotate_head.m[2][1]); */

	/* print the valuator */
	fprintf(stderr," valx/valy %1.4f %1.4f \r",valx,valy);

	/*
	fprintf(stderr," %1.3f %1.3f %1.3f   %1.3f %1.3f %1.3f \r",rotate_hand.m[0][0], rotate_hand.m[1][0], rotate_hand.m[2][0],rotate_hand.m[0][1], rotate_hand.m[1][1], rotate_hand.m[2][1]);
	*/
#if 0
	fprintf(stderr," %1.3f %1.3f %1.3f   %1.3f %1.3f %1.3f \r",hand_orientation[0],hand_orientation[1],hand_orientation[2],rotate_hand.m[0][1], rotate_hand.m[1][1], rotate_hand.m[2][1]);
#endif
	
#if 0
	fprintf(stderr,"%d head: %1.2f %1.2f %1.2f   %1.2f %1.2f %1.2f     hand: %1.2f %1.2f %1.2f  %1.2f %1.2f %1.2f\r",nloops, head_position[0],head_position[1],head_position[2],head_orientation[0],head_orientation[1],head_orientation[2],hand_position[0],hand_position[1],hand_position[2],hand_orientation[0],hand_orientation[1],hand_orientation[2]);

#endif
	if (have_any_buttons)
	{
	    int j;
	    
	    fprintf(stderr,"\n buttons: ");
	    for (j=0;j<nbuttons;j++)
		fprintf(stderr," %d",buttons[j]);
	    fprintf(stderr," \n");
	}
	sginap(sgi_nap_length);
    }

    close_cave();
}
