/*
 * trackd.h source: of unknown origin.
 *
 * 2/16/2000
 * since there is no copyright on this file, i'm going to assume it's
 * ok to redistribute. if there are any questions, concerns, rants
 * or raves, please contact:
 *
 * Wes Bethel, Lawrence Berkeley National Laboratory
 * ewbethel@lbl.gov
 *
 */


#include <inttypes.h>
#include "cave.h"

#ifdef __cplusplus
extern "C" {
#endif

/*    Stuff from CAVElib    */

typedef struct
	{
	uint32_t version;		/* CAVElib version (see constants below) */
	uint32_t numSensors;		/* Total number of sensors */
	uint32_t sensorOffset;		/* Byte offset from of header to start of sensor array */
	uint32_t sensorSize;		/* sizeof() of a sensor struct */
	uint32_t timestamp[2];		/* NB: *Not* a struct timeval - that changes */
					/*	size between 32 & 64 bit mode */
	uint32_t command;		/* For sending commands (such as 'reset') to daemon */
	} CAVE_TRACKDTRACKER_HEADER;


/* Command values for header.command */
/*   header.command should be 0 when no command is pending */
#define CAVE_TRACKD_NO_COMMAND		0
#define CAVE_TRACKD_RESET_COMMAND	1


typedef struct
	{
	uint32_t version;		/* CAVElib version (see constants below) */
	uint32_t buttonOffset;		/* Byte offset from start of header to start of button data */
	uint32_t valuatorOffset;	/* Byte offset from start of header to start of valuator data */
	uint32_t numButtons;
	uint32_t numValuators;
	uint32_t timestamp[2];
	uint32_t command;		/* For sending commands (such as 'reset') to daemon */
	} CAVE_TRACKDCONTROLLER_HEADER;


/* ID Numbers for CAVElib version */
/*  Increasing values should be assigned for newer library versions whenever
    either the header, sensor, or controller struct definition changes */

#define CAVELIB_2_6	1




/* Additional stuff for trackd itself */

#define TRACKD_MAX_SENSORS 8

struct TRACKD_TRACKING
	{
	CAVE_TRACKDTRACKER_HEADER	header;
	CAVE_SENSOR_ST			sensor[TRACKD_MAX_SENSORS];
	};

struct TRACKD_CONTROLLER
	{
	CAVE_TRACKDCONTROLLER_HEADER	header;
	CAVE_CONTROLLER_ST		controller;
	};

#ifdef __cplusplus
}
#endif
